All of lore.kernel.org
 help / color / mirror / Atom feed
From: Marc Zyngier <maz@kernel.org>
To: linux-kernel@vger.kernel.org,
	linux-arm-kernel@lists.infradead.org, linux-acpi@vger.kernel.org
Cc: Thomas Gleixner <tglx@linutronix.de>,
	Mark Rutland <mark.rutland@arm.com>,
	Will Deacon <will@kernel.org>,
	"Rafael J. Wysocki" <rafael@kernel.org>,
	Rob Herring <robh@kernel.org>,
	Saravana Kannan <saravanak@google.com>,
	Greg Kroah-Hartman <gregkh@linuxfoundation.org>,
	Sven Peter <sven@kernel.org>, Janne Grunau <j@jannau.net>,
	Suzuki K Poulose <suzuki.poulose@arm.com>,
	James Clark <james.clark@linaro.org>
Subject: [PATCH v2 04/25] platform: Add firmware-agnostic irq and affinity retrieval interface
Date: Mon, 15 Sep 2025 09:56:41 +0100	[thread overview]
Message-ID: <20250915085702.519996-5-maz@kernel.org> (raw)
In-Reply-To: <20250915085702.519996-1-maz@kernel.org>

Expand platform_get_irq_optional() to also return an affinity if
available, renaming it to platform_get_irq_affinity() in the
process.

platform_get_irq_optional() is preserved with its current semantics
by calling into the new helper with a NULL affinity pointer.

Signed-off-by: Marc Zyngier <maz@kernel.org>
---
 drivers/base/platform.c         | 60 +++++++++++++++++++++++++++------
 include/linux/platform_device.h |  2 ++
 2 files changed, 52 insertions(+), 10 deletions(-)

diff --git a/include/linux/platform_device.h b/include/linux/platform_device.h
index 074754c23d330..ad66333ce85ce 100644
--- a/include/linux/platform_device.h
+++ b/include/linux/platform_device.h
@@ -102,6 +102,8 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 
 extern int platform_get_irq(struct platform_device *, unsigned int);
 extern int platform_get_irq_optional(struct platform_device *, unsigned int);
+extern int platform_get_irq_affinity(struct platform_device *, unsigned int,
+				     const struct cpumask **);
 extern int platform_irq_count(struct platform_device *);
 extern int devm_platform_get_irqs_affinity(struct platform_device *dev,
 					   struct irq_affinity *affd,
diff --git a/drivers/base/platform.c b/drivers/base/platform.c
index 09450349cf323..fad33d6b2349a 100644
--- a/drivers/base/platform.c
+++ b/drivers/base/platform.c
@@ -150,25 +150,37 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 EXPORT_SYMBOL_GPL(devm_platform_ioremap_resource_byname);
 #endif /* CONFIG_HAS_IOMEM */
 
+static const struct cpumask *get_irq_affinity(struct platform_device *dev,
+					      unsigned int num)
+{
+	const struct cpumask *mask = NULL;
+#ifndef CONFIG_SPARC
+	struct fwnode_handle *fwnode = dev_fwnode(&dev->dev);
+
+	if (is_of_node(fwnode))
+		mask = of_irq_get_affinity(to_of_node(fwnode), num);
+	else if (is_acpi_device_node(fwnode))
+		mask = acpi_irq_get_affinity(ACPI_HANDLE_FWNODE(fwnode), num);
+#endif
+
+	return mask ?: cpu_possible_mask;
+}
+
 /**
  * platform_get_irq_optional - get an optional IRQ for a device
  * @dev: platform device
  * @num: IRQ number index
+ * @affinity: optional cpumask pointer to get the affinity of a per-cpu IRQ
  *
  * Gets an IRQ for a platform device. Device drivers should check the return
  * value for errors so as to not pass a negative integer value to the
- * request_irq() APIs. This is the same as platform_get_irq(), except that it
- * does not print an error message if an IRQ can not be obtained.
- *
- * For example::
- *
- *		int irq = platform_get_irq_optional(pdev, 0);
- *		if (irq < 0)
- *			return irq;
+ * request_irq() APIs. Optional affinity information is provided in the
+ * affinity pointer if available, and NULL otherwise.
  *
  * Return: non-zero IRQ number on success, negative error number on failure.
  */
-int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
+int platform_get_irq_affinity(struct platform_device *dev, unsigned int num,
+			      const struct cpumask **affinity)
 {
 	int ret;
 #ifdef CONFIG_SPARC
@@ -236,8 +248,36 @@ int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
 out:
 	if (WARN(!ret, "0 is an invalid IRQ number\n"))
 		return -EINVAL;
+
+	if (ret > 0 && affinity)
+		*affinity = get_irq_affinity(dev, num);
+
 	return ret;
 }
+EXPORT_SYMBOL_GPL(platform_get_irq_affinity);
+
+/**
+ * platform_get_irq_optional - get an optional IRQ for a device
+ * @dev: platform device
+ * @num: IRQ number index
+ *
+ * Gets an IRQ for a platform device. Device drivers should check the return
+ * value for errors so as to not pass a negative integer value to the
+ * request_irq() APIs. This is the same as platform_get_irq(), except that it
+ * does not print an error message if an IRQ can not be obtained.
+ *
+ * For example::
+ *
+ *		int irq = platform_get_irq_optional(pdev, 0);
+ *		if (irq < 0)
+ *			return irq;
+ *
+ * Return: non-zero IRQ number on success, negative error number on failure.
+ */
+int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
+{
+	return platform_get_irq_affinity(dev, num, NULL);
+}
 EXPORT_SYMBOL_GPL(platform_get_irq_optional);
 
 /**
@@ -150,25 +150,37 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 /**
- * platform_get_irq_optional - get an optional IRQ for a device
+ * platform_get_irq_affinity - get an optional IRQ and its affnity for a device
  * @dev: platform device
  * @num: IRQ number index
-- 
2.39.2


  parent reply	other threads:[~2025-09-15  8:57 UTC|newest]

Thread overview: 39+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-09-15  8:56 [PATCH v2 00/25] genirq: Add support for percpu_devid IRQ affinity Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 01/25] irqdomain: Add firmware info reporting interface Marc Zyngier
2025-09-16 15:14   ` Jonathan Cameron
2025-09-16 16:00     ` Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 02/25] ACPI: irq: Add IRQ affinity " Marc Zyngier
2025-09-15 10:16   ` Rafael J. Wysocki
2025-09-15  8:56 ` [PATCH v2 03/25] of/irq: " Marc Zyngier
2025-09-15  8:56 ` Marc Zyngier [this message]
2025-09-16 15:23   ` [PATCH v2 04/25] platform: Add firmware-agnostic irq and affinity retrieval interface Jonathan Cameron
2025-09-15  8:56 ` [PATCH v2 05/25] irqchip/gic-v3: Add FW info retrieval support Marc Zyngier
2025-09-16 15:34   ` Jonathan Cameron
2025-09-16 15:59     ` Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 06/25] irqchip/apple-aic: " Marc Zyngier
2025-09-15 15:07   ` Sven Peter
2025-09-15  8:56 ` [PATCH v2 07/25] coresight: trbe: Convert to new IRQ affinity retrieval API Marc Zyngier
2025-09-15  9:30   ` Suzuki K Poulose
2025-09-15  8:56 ` [PATCH v2 08/25] perf: arm_pmu: " Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 09/25] perf: arm_spe_pmu: " Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 10/25] irqchip/gic-v3: Switch high priority PPIs over to handle_percpu_devid_irq() Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 11/25] genirq: Kill handle_percpu_devid_fasteoi_nmi() Marc Zyngier
2025-09-16 15:38   ` Jonathan Cameron
2025-09-21 15:32     ` Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 12/25] genirq: Merge irqaction::{dev_id,percpu_dev_id} Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 13/25] genirq: Factor-in percpu irqaction creation Marc Zyngier
2025-09-16 16:12   ` Jonathan Cameron
2025-09-15  8:56 ` [PATCH v2 14/25] genirq: Add affinity to percpu_devid interrupt requests Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 15/25] genirq: Update request_percpu_nmi() to take an affinity Marc Zyngier
2025-09-16 16:19   ` Jonathan Cameron
2025-09-15  8:56 ` [PATCH v2 16/25] genirq: Allow per-cpu interrupt sharing for non-overlapping affinities Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 17/25] genirq: Add request_percpu_irq_affinity() helper Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 18/25] perf: arm_pmu: Request specific affinities for percpu NMI/IRQ Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 19/25] perf: arm_spe_pmu: Request specific affinities for percpu IRQ Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 20/25] coresight: trbe: " Marc Zyngier
2025-09-15  9:37   ` Suzuki K Poulose
2025-09-15  8:56 ` [PATCH v2 21/25] irqchip/gic-v3: Drop support for custom PPI partitions Marc Zyngier
2025-09-15  8:56 ` [PATCH v2 22/25] irqchip/apple-aic: Drop support for custom PMU irq partitions Marc Zyngier
2025-09-15  8:57 ` [PATCH v2 23/25] irqchip: Kill irq-partition-percpu Marc Zyngier
2025-09-15  8:57 ` [PATCH v2 24/25] genirq: Kill irq_{g,s}et_percpu_devid_partition() Marc Zyngier
2025-09-15  8:57 ` [PATCH v2 25/25] perf: arm_pmu: Kill last use of per-CPU cpu_armpmu pointer Marc Zyngier

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20250915085702.519996-5-maz@kernel.org \
    --to=maz@kernel.org \
    --cc=gregkh@linuxfoundation.org \
    --cc=j@jannau.net \
    --cc=james.clark@linaro.org \
    --cc=linux-acpi@vger.kernel.org \
    --cc=linux-arm-kernel@lists.infradead.org \
    --cc=linux-kernel@vger.kernel.org \
    --cc=mark.rutland@arm.com \
    --cc=rafael@kernel.org \
    --cc=robh@kernel.org \
    --cc=saravanak@google.com \
    --cc=suzuki.poulose@arm.com \
    --cc=sven@kernel.org \
    --cc=tglx@linutronix.de \
    --cc=will@kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.